-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCommentData")
local gratuity = AceLibrary("Gratuity-2.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Comments"] = true,
	["Comment options"] = true,

	["Save Comment Data"] = true,
	["Stores comment data between sessions"] = true,

	["Automatic Comments"] = true,
	["Automatically set comments from tooltip tags"] = true,

	["   Include Hostile Units"] = true,
	["Include hostile units when automatically gathering comments"] = true,

	["Reset Comments"] = true,
	["Empties the comment database"] = true,
	
	["Level"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Comments"] = "메모",
	["Comment options"] = "메모 설정",

	["Save Comment Data"] = "메모 데이터 저장",
	["Stores comment data between sessions"] = "접속 간 메모 데이터를 저장합니다.",

	["Automatic Comments"] = "자동 메모",
	["Automatically set comments from tooltip tags"] = "자동으로 툴팁 태그를 메모에 설정합니다.",

	["   Include Hostile Units"] = "   적대적 대상 포함",
	["Include hostile units when automatically gathering comments"] = "자동 메모 설정 시 적대적 대상도 포함시킵니다.",

	["Reset Comments"] = "메모 초기화",
	["Empties the comment database"] = "메모 데이터를 삭제합니다.",
	
	["Level"] = "레벨",
} end)

-----------------------------------------------------------------------------

local comments = { }

-----------------------------------------------------------------------------

local AloftCommentData = Aloft:NewModule("CommentData", "AceEvent-2.0")
AloftCommentData.db = Aloft:AcquireDBNamespace("commentData")
Aloft:RegisterDefaults("commentData", "account", { comments = { } })
Aloft:RegisterDefaults("commentData", "profile", { save = true, auto = true, includeHostile = false })

-----------------------------------------------------------------------------

Aloft.Options.args.data.args.comment =
{
	type = 'group',
	name = L["Comments"],
	desc = L["Comment options"],
	args =
	{
		save =
		{
			type = 'toggle',
			name = L["Save Comment Data"],
			desc = L["Stores comment data between sessions"],
			get = function() return AloftCommentData.db.profile.save end,
			order = 1,
			set = function(v)
					AloftCommentData.db.profile.save = v
					if v then
						AloftCommentData.db.server.comments = comments
					else
						AloftCommentData.db.server.comments = comments
					end
				  end,
		},
		auto =
		{
			type = 'toggle',
			name = L["Automatic Comments"],
			desc = L["Automatically set comments from tooltip tags"],
			order = 2,
			get = function() return AloftCommentData.db.profile.auto end,
			set = function(v)
					AloftCommentData.db.profile.auto = v
					AloftCommentData:RegisterEvents()
				  end,
		},
		includeHostile =
		{
			type = 'toggle',
			name = L["   Include Hostile Units"],
			desc = L["Include hostile units when automatically gathering comments"],
			order = 3,
			disabled = function() return not AloftCommentData.db.profile.auto end,
			get = function() return AloftCommentData.db.profile.includeHostile end,
			set = function(v) AloftCommentData.db.profile.includeHostile = v end,
		},
		clear =
		{
			type = "execute",
			name = L["Reset Comments"],
			desc = L["Empties the comment database"],
			func = function() AloftCommentData:ClearAllData() end,
			order = -1,
		},
	}
}

-----------------------------------------------------------------------------

Aloft.TagData.comment =
{
	data = "comment",
	events = "Aloft:OnCommentDataChanged",
	tableData = "comment"
}

-----------------------------------------------------------------------------

function AloftCommentData:ProvidesData()
	return "comment"
end

function AloftCommentData:RegisterEvents()
	self:UnregisterAllEvents()
	if self.db.profile.auto then
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdateCommentFromMouseOver")
		self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateCommentFromTarget")
	end
end

function AloftCommentData:EnableDataSource()
	self:RegisterEvents()
end

-----------------------------------------------------------------------------

function AloftCommentData:OnEnable()
	if self.db.profile.save then
		comments = self.db.account.comments
	end
end

function AloftCommentData:UpdateData(aloftData)
	aloftData.comment = comments[aloftData.name]
end

-----------------------------------------------------------------------------

function AloftCommentData:UpdateAll()
	for aloftData in Aloft:IterateVisibleNameplates() do
		local comment = comments[aloftData.name]
		if aloftData.comment ~= comment then
			aloftData.comment = comment
			self:TriggerEvent("Aloft:OnCommentDataChanged", aloftData)
		end
	end
end

function AloftCommentData:SetCommentForTarget(comment)
	if UnitExists("target") then
		local name = UnitName("target")
		comments[name] = comment ~= "" and comment or nil
		self:UpdateAll()
		return name
	end
end

function AloftCommentData:ClearAllData()
	for k in pairs(comments) do
		comments[k] = nil
	end
	self:UpdateAll()
end

-----------------------------------------------------------------------------

function AloftCommentData:UpdateCommentFromUnit(unitid)
	if UnitExists(unitid) and not UnitPlayerControlled(unitid) then

		if not self.db.profile.includeHostile and not UnitIsFriend("player", unitid) then 
			return
		end

		local key = UnitName(unitid)
		
		if not comments[key] then
			gratuity:SetUnit(unitid)
			local comment = gratuity:GetLine(2)

			if comment and comment ~= "" and comment:find(L["Level"]) ~= 1 then
				comments[key] = comment

				for aloftData in Aloft:IterateVisibleNameplates() do
					if aloftData.name == key then
						aloftData.comment = comment
						self:TriggerEvent("Aloft:OnCommentDataChanged", aloftData)
					end
				end
			end

		end
	end
end

function AloftCommentData:UpdateCommentFromMouseOver()
	self:UpdateCommentFromUnit("mouseover")
end

function AloftCommentData:UpdateCommentFromTarget()
	self:UpdateCommentFromUnit("target")
end

-----------------------------------------------------------------------------
